@@ -17,15 +17,11 @@ import logging
 
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
-
 from keystoneclient import exceptions as keystone_exceptions
 
-from .exceptions import KeystoneAuthException
-from .user import create_user_from_token
-from .user import Token
-from .utils import check_token_expiration
-from .utils import get_keystone_client
-from .utils import get_keystone_version
+from openstack_auth import exceptions
+from openstack_auth import user as auth_user
+from openstack_auth import utils
 
 
 LOG = logging.getLogger(__name__)
@@ -39,15 +35,15 @@ class KeystoneBackend(object):
       ``django.contrib.auth``.
     """
 
-    def check_auth_expiry(self, auth_ref):
-        if not check_token_expiration(auth_ref):
+    def check_auth_expiry(self, auth_ref, margin=None):
+        if not utils.is_token_valid(auth_ref, margin):
             msg = _("The authentication token issued by the Identity service "
                     "has expired.")
             LOG.warning("The authentication token issued by the Identity "
                         "service appears to have expired before it was "
                         "issued. This may indicate a problem with either your "
                         "server or client configuration.")
-            raise KeystoneAuthException(msg)
+            raise exceptions.KeystoneAuthException(msg)
         return True
 
     def get_user(self, user_id):
@@ -62,15 +58,15 @@ class KeystoneBackend(object):
             token = self.request.session['token']
             endpoint = self.request.session['region_endpoint']
             services_region = self.request.session['services_region']
-            user = create_user_from_token(self.request, token, endpoint,
-                                          services_region)
+            user = auth_user.create_user_from_token(self.request, token,
+                                                    endpoint, services_region)
             return user
         else:
             return None
 
     def authenticate(self, request=None, username=None, password=None,
                      user_domain_name=None, auth_url=None):
-        """Authenticates a user via the Keystone Identity API. """
+        """Authenticates a user via the Keystone Identity API."""
         LOG.debug('Beginning user authentication for user "%s".' % username)
 
         insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
@@ -78,11 +74,18 @@ class KeystoneBackend(object):
         endpoint_type = getattr(
             settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')
 
-        # keystone client v3 does not support logging in on the v2 url any more
-        if get_keystone_version() >= 3:
-            auth_url = auth_url.replace('v2.0', 'v3')
+        if auth_url is None:
+            auth_url = settings.OPENSTACK_KEYSTONE_URL
 
-        keystone_client = get_keystone_client()
+        # keystone client v3 does not support logging in on the v2 url any more
+        if utils.get_keystone_version() >= 3:
+            if utils.has_in_url_path(auth_url, "/v2.0"):
+                LOG.warning("The settings.py file points to a v2.0 keystone "
+                            "endpoint, but v3 is specified as the API version "
+                            "to use. Using v3 endpoint for authentication.")
+                auth_url = utils.url_path_replace(auth_url, "/v2.0", "/v3", 1)
+
+        keystone_client = utils.get_keystone_client()
         try:
             client = keystone_client.Client(
                 user_domain_name=user_domain_name,
@@ -94,72 +97,89 @@ class KeystoneBackend(object):
                 debug=settings.DEBUG)
 
             unscoped_auth_ref = client.auth_ref
-            unscoped_token = Token(auth_ref=unscoped_auth_ref)
+            unscoped_token = auth_user.Token(auth_ref=unscoped_auth_ref)
         except (keystone_exceptions.Unauthorized,
                 keystone_exceptions.Forbidden,
                 keystone_exceptions.NotFound) as exc:
             msg = _('Invalid user name or password.')
-            LOG.debug(exc.message)
-            raise KeystoneAuthException(msg)
+            LOG.debug(str(exc))
+            raise exceptions.KeystoneAuthException(msg)
         except (keystone_exceptions.ClientException,
                 keystone_exceptions.AuthorizationFailure) as exc:
             msg = _("An error occurred authenticating. "
                     "Please try again later.")
-            LOG.debug(exc.message)
-            raise KeystoneAuthException(msg)
+            LOG.debug(str(exc))
+            raise exceptions.KeystoneAuthException(msg)
 
         # Check expiry for our unscoped auth ref.
         self.check_auth_expiry(unscoped_auth_ref)
 
         # Check if token is automatically scoped to default_project
+        # grab the project from this token, to use as a default
+        # if no recent_project is found in the cookie
+        token_proj_id = None
         if unscoped_auth_ref.project_scoped:
-            auth_ref = unscoped_auth_ref
-        else:
-            # For now we list all the user's projects and iterate through.
+            token_proj_id = unscoped_auth_ref.get('project',
+                                                  {}).get('id')
+
+        # We list all the user's projects
+        try:
+            if utils.get_keystone_version() < 3:
+                projects = client.tenants.list()
+            else:
+                client.management_url = auth_url
+                projects = client.projects.list(
+                    user=unscoped_auth_ref.user_id)
+        except (keystone_exceptions.ClientException,
+                keystone_exceptions.AuthorizationFailure) as exc:
+            msg = _('Unable to retrieve authorized projects.')
+            raise exceptions.KeystoneAuthException(msg)
+
+        # Abort if there are no projects for this user
+        if not projects:
+            msg = _('You are not authorized for any projects.')
+            raise exceptions.KeystoneAuthException(msg)
+
+        # the recent project id a user might have set in a cookie
+        recent_project = None
+        if request:
+            recent_project = request.COOKIES.get('recent_project',
+                                                 token_proj_id)
+
+        # if a most recent project was found, try using it first
+        for pos, project in enumerate(projects):
+            if project.id == recent_project:
+                # move recent project to the beginning
+                projects.pop(pos)
+                projects.insert(0, project)
+                break
+
+        for project in projects:
             try:
-                if get_keystone_version() < 3:
-                    projects = client.tenants.list()
-                else:
-                    client.management_url = auth_url
-                    projects = client.projects.list(
-                        user=unscoped_auth_ref.user_id)
+                client = keystone_client.Client(
+                    tenant_id=project.id,
+                    token=unscoped_auth_ref.auth_token,
+                    auth_url=auth_url,
+                    insecure=insecure,
+                    cacert=ca_cert,
+                    debug=settings.DEBUG)
+                auth_ref = client.auth_ref
+                break
             except (keystone_exceptions.ClientException,
-                    keystone_exceptions.AuthorizationFailure) as exc:
-                msg = _('Unable to retrieve authorized projects.')
-                raise KeystoneAuthException(msg)
+                    keystone_exceptions.AuthorizationFailure):
+                auth_ref = None
 
-            # Abort if there are no projects for this user
-            if not projects:
-                msg = _('You are not authorized for any projects.')
-                raise KeystoneAuthException(msg)
-
-            while projects:
-                project = projects.pop()
-                try:
-                    client = keystone_client.Client(
-                        tenant_id=project.id,
-                        token=unscoped_auth_ref.auth_token,
-                        auth_url=auth_url,
-                        insecure=insecure,
-                        cacert=ca_cert,
-                        debug=settings.DEBUG)
-                    auth_ref = client.auth_ref
-                    break
-                except (keystone_exceptions.ClientException,
-                        keystone_exceptions.AuthorizationFailure):
-                    auth_ref = None
-
-            if auth_ref is None:
-                msg = _("Unable to authenticate to any available projects.")
-                raise KeystoneAuthException(msg)
+        if auth_ref is None:
+            msg = _("Unable to authenticate to any available projects.")
+            raise exceptions.KeystoneAuthException(msg)
 
         # Check expiry for our new scoped token.
         self.check_auth_expiry(auth_ref)
 
         # If we made it here we succeeded. Create our User!
-        user = create_user_from_token(
+        user = auth_user.create_user_from_token(
             request,
-            Token(auth_ref),
+            auth_user.Token(auth_ref),
             client.service_catalog.url_for(endpoint_type=endpoint_type))
 
         if request is not None:
@@ -173,7 +193,7 @@ class KeystoneBackend(object):
         return user
 
     def get_group_permissions(self, user, obj=None):
-        """Returns an empty set since Keystone doesn't support "groups". """
+        """Returns an empty set since Keystone doesn't support "groups"."""
         # Keystone V3 added "groups". The Auth token response includes the
         # roles from the user's Group assignment. It should be fine just
         # returning an empty set here.
@@ -192,11 +212,14 @@ class KeystoneBackend(object):
         role_perms = set(["openstack.roles.%s" % role['name'].lower()
                           for role in user.roles])
         service_perms = set(["openstack.services.%s" % service['type'].lower()
-                          for service in user.service_catalog])
+                             for service in user.service_catalog
+                             if user.services_region in
+                             [endpoint.get('region', None) for endpoint
+                              in service.get('endpoints', [])]])
         return role_perms | service_perms
 
     def has_perm(self, user, perm, obj=None):
-        """Returns True if the given user has the specified permission. """
+        """Returns True if the given user has the specified permission."""
         if not user.is_active:
             return False
         return perm in self.get_all_permissions(user, obj)
