@@ -13,21 +13,21 @@
 
 import logging
 
-from django import forms
 from django.conf import settings
-from django.contrib.auth import authenticate
-from django.contrib.auth.forms import AuthenticationForm
+from django.contrib.auth import authenticate  # noqa
+from django.contrib.auth import forms as django_auth_forms
+from django import forms
 from django.utils.translation import ugettext_lazy as _
-from django.views.decorators.debug import sensitive_variables
+from django.views.decorators.debug import sensitive_variables  # noqa
 
-from .exceptions import KeystoneAuthException
+from openstack_auth import exceptions
 
 
 LOG = logging.getLogger(__name__)
 
 
-class Login(AuthenticationForm):
-    """ Form used for logging in a user.
+class Login(django_auth_forms.AuthenticationForm):
+    """Form used for logging in a user.
 
     Handles authentication with Keystone by providing the domain name, username
     and password. A scoped token is fetched after successful authentication.
@@ -57,7 +57,7 @@ class Login(AuthenticationForm):
         self.fields.keyOrder = ['username', 'password', 'region']
         if getattr(settings,
                    'OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT',
-                    False):
+                   False):
             self.fields['domain'] = forms.CharField(label=_("Domain"),
                                                     required=True)
             self.fields.keyOrder = ['domain', 'username', 'password', 'region']
@@ -94,11 +94,11 @@ class Login(AuthenticationForm):
             msg = 'Login successful for user "%(username)s".' % \
                 {'username': username}
             LOG.info(msg)
-        except KeystoneAuthException as exc:
+        except exceptions.KeystoneAuthException as exc:
             msg = 'Login failed for user "%(username)s".' % \
                 {'username': username}
             LOG.warning(msg)
-            self.request.session.flush()
             raise forms.ValidationError(exc)
-        self.check_for_test_cookie()
+        if hasattr(self, 'check_for_test_cookie'):  # Dropped in django 1.7
+            self.check_for_test_cookie()
         return self.cleaned_data
