@@ -10,33 +10,34 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 import logging
 
 import django
-from django import shortcuts
 from django.conf import settings
-from django.contrib.auth import REDIRECT_FIELD_NAME
-from django.contrib.auth.views import (login as django_login,
-                                       logout_then_login as django_logout)
-from django.contrib.auth.decorators import login_required
-from django.views.decorators.debug import sensitive_post_parameters
-from django.utils.functional import curry
-from django.views.decorators.cache import never_cache
-from django.views.decorators.csrf import csrf_protect
+from django.contrib import auth
+from django.contrib.auth.decorators import login_required  # noqa
+from django.contrib.auth import views as django_auth_views
+from django import shortcuts
+from django.utils import functional
+from django.utils import http
+from django.views.decorators.cache import never_cache  # noqa
+from django.views.decorators.csrf import csrf_protect  # noqa
+from django.views.decorators.debug import sensitive_post_parameters  # noqa
+from keystoneclient import exceptions as keystone_exceptions
+from keystoneclient.v2_0 import client as keystone_client_v2
+
+from openstack_auth import forms
+# This is historic and is added back in to not break older versions of
+# Horizon, fix to Horizon to remove this requirement was committed in
+# Juno
+from openstack_auth.forms import Login  # noqa
+from openstack_auth import user as auth_user
+from openstack_auth import utils
 
 try:
-    from django.utils.http import is_safe_url
-except ImportError:
-    from .utils import is_safe_url
-
-from keystoneclient.v2_0 import client as keystone_client_v2
-from keystoneclient import exceptions as keystone_exceptions
-
-from .forms import Login
-from .user import set_session_from_user, create_user_from_token, Token
-from .utils import get_keystone_client
-from .utils import get_keystone_version
+    is_safe_url = http.is_safe_url
+except AttributeError:
+    is_safe_url = utils.is_safe_url
 
 
 LOG = logging.getLogger(__name__)
@@ -45,8 +46,18 @@ LOG = logging.getLogger(__name__)
 @sensitive_post_parameters()
 @csrf_protect
 @never_cache
-def login(request):
-    """ Logs a user in using the :class:`~openstack_auth.forms.Login` form. """
+def login(request, template_name=None, extra_context=None, **kwargs):
+    """Logs a user in using the :class:`~openstack_auth.forms.Login` form."""
+    if not request.is_ajax():
+        # If the user is already authenticated, redirect them to the
+        # dashboard straight away, unless the 'next' parameter is set as it
+        # usually indicates requesting access to a page that requires different
+        # permissions.
+        if (request.user.is_authenticated() and
+                auth.REDIRECT_FIELD_NAME not in request.GET and
+                auth.REDIRECT_FIELD_NAME not in request.POST):
+            return shortcuts.redirect(settings.LOGIN_REDIRECT_URL)
+
     # Get our initial region for the form.
     initial = {}
     current_region = request.session.get('region_endpoint', None)
@@ -60,29 +71,32 @@ def login(request):
         # the 'request' object is passed directly to AuthenticationForm in
         # django.contrib.auth.views#login:
         if django.VERSION >= (1, 6):
-            form = curry(Login)
+            form = functional.curry(forms.Login)
         else:
-            form = curry(Login, request)
+            form = functional.curry(forms.Login, request)
     else:
-        form = curry(Login, initial=initial)
+        form = functional.curry(forms.Login, initial=initial)
 
-    extra_context = {'redirect_field_name': REDIRECT_FIELD_NAME}
+    if extra_context is None:
+        extra_context = {'redirect_field_name': auth.REDIRECT_FIELD_NAME}
 
-    if request.is_ajax():
-        template_name = 'auth/_login.html'
-        extra_context['hide'] = True
-    else:
-        template_name = 'auth/login.html'
+    if not template_name:
+        if request.is_ajax():
+            template_name = 'auth/_login.html'
+            extra_context['hide'] = True
+        else:
+            template_name = 'auth/login.html'
 
-    res = django_login(request,
-                       template_name=template_name,
-                       authentication_form=form,
-                       extra_context=extra_context)
+    res = django_auth_views.login(request,
+                                  template_name=template_name,
+                                  authentication_form=form,
+                                  extra_context=extra_context,
+                                  **kwargs)
     # Set the session data here because django's session key rotation
     # will erase it if we set it earlier.
     if request.user.is_authenticated():
-        set_session_from_user(request, request.user)
-        regions = dict(Login.get_region_choices())
+        auth_user.set_session_from_user(request, request.user)
+        regions = dict(forms.Login.get_region_choices())
         region = request.user.endpoint
         region_name = regions.get(region)
         request.session['region_endpoint'] = region
@@ -90,7 +104,17 @@ def login(request):
     return res
 
 
-def logout(request):
+def logout(request, login_url=None, **kwargs):
+    """Logs out the user if he is logged in. Then redirects to the log-in page.
+
+    .. param:: login_url
+
+       Once logged out, defines the URL where to redirect after login
+
+    .. param:: kwargs
+       see django.contrib.auth.views.logout_then_login extra parameters.
+
+    """
     msg = 'Logging out user "%(username)s".' % \
         {'username': request.user.username}
     LOG.info(msg)
@@ -99,7 +123,8 @@ def logout(request):
     if token and endpoint:
         delete_token(endpoint=endpoint, token_id=token.id)
     """ Securely logs a user out. """
-    return django_logout(request)
+    return django_auth_views.logout_then_login(request, login_url=login_url,
+                                               **kwargs)
 
 
 def delete_token(endpoint, token_id):
@@ -107,8 +132,9 @@ def delete_token(endpoint, token_id):
 
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
+    utils.remove_project_cache(token_id)
     try:
-        if get_keystone_version() < 3:
+        if utils.get_keystone_version() < 3:
             client = keystone_client_v2.Client(
                 endpoint=endpoint,
                 token=token_id,
@@ -122,28 +148,29 @@ def delete_token(endpoint, token_id):
             # FIXME: KS-client does not have delete token available
             # Need to add this later when it is exposed.
             pass
-    except keystone_exceptions.ClientException as e:
+    except keystone_exceptions.ClientException:
         LOG.info('Could not delete token')
 
 
 @login_required
-def switch(request, tenant_id, redirect_field_name=REDIRECT_FIELD_NAME):
-    """ Switches an authenticated user from one project to another. """
+def switch(request, tenant_id, redirect_field_name=auth.REDIRECT_FIELD_NAME):
+    """Switches an authenticated user from one project to another."""
     LOG.debug('Switching to tenant %s for user "%s".'
               % (tenant_id, request.user.username))
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
     endpoint = request.user.endpoint
     try:
-        if get_keystone_version() >= 3:
-            if 'v3' not in endpoint:
-                endpoint = endpoint.replace('v2.0', 'v3')
-        client = get_keystone_client().Client(tenant_id=tenant_id,
-                                              token=request.user.token.id,
-                                              auth_url=endpoint,
-                                              insecure=insecure,
-                                              cacert=ca_cert,
-                                              debug=settings.DEBUG)
+        if utils.get_keystone_version() >= 3:
+            if not utils.has_in_url_path(endpoint, '/v3'):
+                endpoint = utils.url_path_replace(endpoint, '/v2.0', '/v3', 1)
+        client = utils.get_keystone_client().Client(
+            tenant_id=tenant_id,
+            token=request.user.token.id,
+            auth_url=endpoint,
+            insecure=insecure,
+            cacert=ca_cert,
+            debug=settings.DEBUG)
         auth_ref = client.auth_ref
         msg = 'Project switch successful for user "%(username)s".' % \
             {'username': request.user.username}
@@ -166,17 +193,22 @@ def switch(request, tenant_id, redirect_field_name=REDIRECT_FIELD_NAME):
         old_token = request.session.get('token')
         if old_token and old_endpoint and old_token.id != auth_ref.auth_token:
             delete_token(endpoint=old_endpoint, token_id=old_token.id)
-        user = create_user_from_token(request, Token(auth_ref), endpoint)
-        set_session_from_user(request, user)
-    return shortcuts.redirect(redirect_to)
+        user = auth_user.create_user_from_token(
+            request, auth_user.Token(auth_ref), endpoint)
+        auth_user.set_session_from_user(request, user)
+    response = shortcuts.redirect(redirect_to)
+    utils.set_response_cookie(response, 'recent_project',
+                              request.user.project_id)
+    return response
 
 
 @login_required
 def switch_region(request, region_name,
-                  redirect_field_name=REDIRECT_FIELD_NAME):
-    """
-    Switches the non-identity services region that is being managed
-    for the scoped project.
+                  redirect_field_name=auth.REDIRECT_FIELD_NAME):
+    """Switches the user's region for all services except Identity service.
+
+    The region will be switched if the given region is one of the regions
+    available for the scoped project. Otherwise the region is not switched.
     """
     if region_name in request.user.available_services_regions:
         request.session['services_region'] = region_name
@@ -187,4 +219,7 @@ def switch_region(request, region_name,
     if not is_safe_url(url=redirect_to, host=request.get_host()):
         redirect_to = settings.LOGIN_REDIRECT_URL
 
-    return shortcuts.redirect(redirect_to)
+    response = shortcuts.redirect(redirect_to)
+    utils.set_response_cookie(response, 'services_region',
+                              request.session['services_region'])
+    return response
